var Phobos, chalk, fs, fsutil, http, lodash, os, path, queryParse, response, urlParse;

fs = require('fs');

http = require('http');

lodash = require('lodash');

fsutil = require('nodejs-fs-utils');

path = require('path');

urlParse = require('url').parse;

queryParse = require('querystring').parse;

os = require('mars-deimos').os;

response = require('./response');

chalk = require('chalk');

Phobos = (function() {
  function Phobos(opt) {
    this.cwd = process.cwd();
    this.rewrite = [];
    this.initOptions(opt).parseRewrite();
    if (this.options.dir[0] !== '/') {
      this.dir = path.join(this.cwd, this.options.dir);
    }
    this.response = response(this.options.vars, {
      locale_data: this.options.locale_data,
      locale: this.options.locale
    });
  }

  Phobos.prototype.initOptions = function(opt) {
    var e, error, options, rc, rcOptions;
    options = {
      dir: 'phobos',
      locale: 'zh_CN',
      locale_data: {},
      rewrite: [],
      vars: {}
    };
    rc = path.join(this.cwd, '.phobosrc');
    if (fs.existsSync(rc)) {
      try {
        rcOptions = require(rc);
      } catch (error) {
        e = error;
        console.error(("[" + (chalk.bold.red('phobos rcfile parse fail')) + "] ") + ("\"" + (path.relative(this.cwd, rc)) + "\" " + (chalk.gray(e.message))));
      }
      options = os(options, rcOptions);
    }
    this.options = os(options, opt);
    return this;
  };

  Phobos.prototype.parseRewrite = function() {
    var i, len, ref, rule;
    ref = this.options.rewrite;
    for (i = 0, len = ref.length; i < len; i++) {
      rule = ref[i];
      if ('string' === typeof rule.test) {
        rule = lodash.clone(rule);
        rule.test = new RegExp("^" + (rule.test.replace(/\*/g, '(.*)')) + "$");
      }
      this.rewrite.push(rule);
    }
    return this;
  };

  Phobos.prototype.middleware = function() {
    var mw;
    mw = (function(_this) {
      return function(req, resp, next) {
        var chunks, method, run;
        method = req.method.toUpperCase();
        run = function(post) {
          var data, url, urlRewrite;
          url = req.url;
          if (urlRewrite = _this.routerRewrite(req.url, method)) {
            if (/^http:\/\/.+/.test(req.url)) {
              req.proxyToUrl = urlRewrite;
            } else {
              req.ordinaryUrl = url;
              req.url = url = urlRewrite;
            }
          }
          if (_this.routerProxy(req, resp, next)) {
            return next();
          }
          if ((data = _this.routerApi(url, method)) == null) {
            return next();
          }
          resp.writeHead(200, {
            'Content-Type': 'application/json; charset=utf-8'
          });
          resp.end(JSON.stringify(_this.response.trans(data, req.url, post)));
        };
        if (method === 'POST' || method === 'PUT') {
          chunks = [];
          req.on('data', function(chunk) {
            return chunks.push(chunk);
          });
          return req.on('end', function() {
            var e, error, post, raw;
            raw = Buffer.concat(chunks).toString();
            post = {};
            try {
              post = JSON.parse(raw);
            } catch (error) {
              e = error;
              post = queryParse(raw);
            }
            return run(post);
          });
        } else {
          return run({});
        }
      };
    })(this);
    mw.phobos = this;
    return mw;
  };

  Phobos.prototype.routerApi = function(url, method) {
    var data, e, error, file, fileAll, fileMethod, json, pathname;
    method = method.toLowerCase();
    if (method !== 'get' && method !== 'post' && method !== 'put' && method !== 'delete') {
      return;
    }
    pathname = urlParse(url).pathname;
    pathname = pathname.replace(/^\/+/g, '');
    fileMethod = path.join(this.dir, pathname, (method.toLowerCase()) + ".json");
    fileAll = path.join(this.dir, pathname + ".json");
    if (fs.existsSync(fileMethod)) {
      file = fileMethod;
    } else if (fs.existsSync(fileAll)) {
      file = fileAll;
    } else {
      return;
    }
    try {
      json = fs.readFileSync(file).toString();
      file = path.relative(this.cwd, file);
      data = JSON.parse(json);
    } catch (error) {
      e = error;
      console.warn(("[" + (chalk.bold.yellow('phobos define-file parse fail')) + "] ") + ("\"" + file + "\" " + (chalk.gray(e.message))));
    }
    return data;
  };

  Phobos.prototype.routerRewrite = function(url, reqMethod) {
    var i, len, method, ref, ref1, target, test;
    ref = this.rewrite;
    for (i = 0, len = ref.length; i < len; i++) {
      ref1 = ref[i], test = ref1.test, target = ref1.target, method = ref1.method;
      method = method || 'all';
      if ((method === 'all' || reqMethod === method) && test.test(url)) {
        return url.replace(test, target);
      }
    }
  };

  Phobos.prototype.routerProxy = function(req, resp, next) {
    var conn, k, options, ref, v;
    if (!req.proxyToUrl) {
      return;
    }
    req.pause();
    options = urlParse(req.url);
    req.url = req.ordinaryUrl;
    options.headers = {};
    ref = req.headers;
    for (k in ref) {
      v = ref[k];
      if (k !== 'host') {
        options.headers[k] = v;
      }
    }
    options.method = req.method;
    options.agent = false;
    conn = http.request(options, function(serverResp) {
      serverResp.pause();
      resp.writeHead(serverResp.statusCode, serverResp.headers);
      serverResp.pipe(resp);
      return serverResp.resume();
    });
    req.pipe(conn);
    req.resume();
    return true;
  };

  return Phobos;

})();

module.exports = function(opt) {
  return new Phobos(opt).middleware();
};

module.exports.Phobos = Phobos;
